@@ -26,6 +26,7 @@
 /*	gid_t	var_default_gid;
 /*	char	*var_config_dir;
 /*	char	*var_daemon_dir;
+/*	char	*var_data_dir;
 /*	char	*var_command_dir;
 /*	char	*var_queue_dir;
 /*	int	var_use_limit;
@@ -73,6 +74,7 @@
 /*	char	*var_par_dom_match;
 /*	char	*var_config_dirs;
 /*
+/*	int	var_inet_windowsize;
 /*	char	*var_import_environ;
 /*	char	*var_export_environ;
 /*	char	*var_debug_peer_list;
@@ -90,6 +92,8 @@
 /*	char   *var_flush_service;
 /*	char   *var_verify_service;
 /*	char   *var_trace_service;
+/*	char   *var_proxymap_service;
+/*	char   *var_proxywrite_service;
 /*	int	var_db_create_buf;
 /*	int	var_db_read_buf;
 /*	int	var_mime_maxdepth;
@@ -108,6 +112,12 @@
 /*	char	*var_int_filt_classes;
 /*	int	var_cyrus_sasl_authzid;
 /*
+/*	char	*var_multi_conf_dirs;
+/*	char	*var_multi_wrapper;
+/*	char	*var_multi_group;
+/*	char	*var_multi_name;
+/*	bool	var_multi_enable;
+/*
 /*	void	mail_params_init()
 /*
 /*	const	char null_format_string[1];
@@ -166,6 +176,7 @@
 #endif
 #include <inet_proto.h>
 #include <vstring_vstream.h>
+#include <iostuff.h>
 
 /* Global library. */
 
@@ -200,6 +211,7 @@ uid_t   var_default_uid;
 gid_t   var_default_gid;
 char   *var_config_dir;
 char   *var_daemon_dir;
+char   *var_data_dir;
 char   *var_command_dir;
 char   *var_queue_dir;
 int     var_use_limit;
@@ -248,6 +260,7 @@ int     var_in_flow_delay;
 char   *var_par_dom_match;
 char   *var_config_dirs;
 
+int     var_inet_windowsize;
 char   *var_import_environ;
 char   *var_export_environ;
 char   *var_debug_peer_list;
@@ -264,6 +277,8 @@ char   *var_error_service;
 char   *var_flush_service;
 char   *var_verify_service;
 char   *var_trace_service;
+char   *var_proxymap_service;
+char   *var_proxywrite_service;
 int     var_db_create_buf;
 int     var_db_read_buf;
 int     var_mime_maxdepth;
@@ -282,6 +297,12 @@ int     var_delay_max_res;
 char   *var_int_filt_classes;
 int     var_cyrus_sasl_authzid;
 
+char   *var_multi_conf_dirs;
+char   *var_multi_wrapper;
+char   *var_multi_group;
+char   *var_multi_name;
+bool    var_multi_enable;
+
 const char null_format_string[1] = "";
 
 /* check_myhostname - lookup hostname and validate */
@@ -485,17 +506,21 @@ static char *read_param_from_file(const char *path)
 
 void    mail_params_init()
 {
-    static CONFIG_STR_TABLE first_str_defaults[] = {
+    static const CONFIG_STR_TABLE first_str_defaults[] = {
 	VAR_SYSLOG_FACILITY, DEF_SYSLOG_FACILITY, &var_syslog_facility, 1, 0,
 	VAR_INET_PROTOCOLS, DEF_INET_PROTOCOLS, &var_inet_protocols, 1, 0,
+	VAR_MULTI_CONF_DIRS, DEF_MULTI_CONF_DIRS, &var_multi_conf_dirs, 0, 0,
+	VAR_MULTI_WRAPPER, DEF_MULTI_WRAPPER, &var_multi_wrapper, 0, 0,
+	VAR_MULTI_GROUP, DEF_MULTI_GROUP, &var_multi_group, 0, 0,
+	VAR_MULTI_NAME, DEF_MULTI_NAME, &var_multi_name, 0, 0,
 	0,
     };
-    static CONFIG_STR_FN_TABLE function_str_defaults[] = {
+    static const CONFIG_STR_FN_TABLE function_str_defaults[] = {
 	VAR_MYHOSTNAME, check_myhostname, &var_myhostname, 1, 0,
 	VAR_MYDOMAIN, check_mydomainname, &var_mydomain, 1, 0,
 	0,
     };
-    static CONFIG_STR_TABLE other_str_defaults[] = {
+    static const CONFIG_STR_TABLE other_str_defaults[] = {
 	VAR_MAIL_NAME, DEF_MAIL_NAME, &var_mail_name, 1, 0,
 	VAR_SYSLOG_NAME, DEF_SYSLOG_NAME, &var_syslog_name, 1, 0,
 	VAR_MAIL_OWNER, DEF_MAIL_OWNER, &var_mail_owner, 1, 0,
@@ -504,6 +529,7 @@ void    mail_params_init()
 	VAR_MYORIGIN, DEF_MYORIGIN, &var_myorigin, 1, 0,
 	VAR_RELAYHOST, DEF_RELAYHOST, &var_relayhost, 0, 0,
 	VAR_DAEMON_DIR, DEF_DAEMON_DIR, &var_daemon_dir, 1, 0,
+	VAR_DATA_DIR, DEF_DATA_DIR, &var_data_dir, 1, 0,
 	VAR_COMMAND_DIR, DEF_COMMAND_DIR, &var_command_dir, 1, 0,
 	VAR_QUEUE_DIR, DEF_QUEUE_DIR, &var_queue_dir, 1, 0,
 	VAR_PID_DIR, DEF_PID_DIR, &var_pid_dir, 1, 0,
@@ -538,14 +564,16 @@ void    mail_params_init()
 	VAR_FLUSH_SERVICE, DEF_FLUSH_SERVICE, &var_flush_service, 1, 0,
 	VAR_VERIFY_SERVICE, DEF_VERIFY_SERVICE, &var_verify_service, 1, 0,
 	VAR_TRACE_SERVICE, DEF_TRACE_SERVICE, &var_trace_service, 1, 0,
+	VAR_PROXYMAP_SERVICE, DEF_PROXYMAP_SERVICE, &var_proxymap_service, 1, 0,
+	VAR_PROXYWRITE_SERVICE, DEF_PROXYWRITE_SERVICE, &var_proxywrite_service, 1, 0,
 	VAR_INT_FILT_CLASSES, DEF_INT_FILT_CLASSES, &var_int_filt_classes, 0, 0,
 	0,
     };
-    static CONFIG_STR_FN_TABLE function_str_defaults_2[] = {
+    static const CONFIG_STR_FN_TABLE function_str_defaults_2[] = {
 	VAR_MYNETWORKS, mynetworks, &var_mynetworks, 0, 0,
 	0,
     };
-    static CONFIG_INT_TABLE other_int_defaults[] = {
+    static const CONFIG_INT_TABLE other_int_defaults[] = {
 	VAR_MAX_USE, DEF_MAX_USE, &var_use_limit, 1, 0,
 	VAR_DONT_REMOVE, DEF_DONT_REMOVE, &var_dont_remove, 0, 0,
 	VAR_LINE_LIMIT, DEF_LINE_LIMIT, &var_line_limit, 512, 0,
@@ -562,9 +590,10 @@ void    mail_params_init()
 	VAR_MIME_MAXDEPTH, DEF_MIME_MAXDEPTH, &var_mime_maxdepth, 1, 0,
 	VAR_MIME_BOUND_LEN, DEF_MIME_BOUND_LEN, &var_mime_bound_len, 1, 0,
 	VAR_DELAY_MAX_RES, DEF_DELAY_MAX_RES, &var_delay_max_res, MIN_DELAY_MAX_RES, MAX_DELAY_MAX_RES,
+	VAR_INET_WINDOW, DEF_INET_WINDOW, &var_inet_windowsize, 0, 0,
 	0,
     };
-    static CONFIG_TIME_TABLE time_defaults[] = {
+    static const CONFIG_TIME_TABLE time_defaults[] = {
 	VAR_EVENT_DRAIN, DEF_EVENT_DRAIN, &var_event_drain, 1, 0,
 	VAR_MAX_IDLE, DEF_MAX_IDLE, &var_idle_limit, 1, 0,
 	VAR_IPC_TIMEOUT, DEF_IPC_TIMEOUT, &var_ipc_timeout, 1, 0,
@@ -578,7 +607,7 @@ void    mail_params_init()
 	VAR_IN_FLOW_DELAY, DEF_IN_FLOW_DELAY, &var_in_flow_delay, 0, 10,
 	0,
     };
-    static CONFIG_BOOL_TABLE bool_defaults[] = {
+    static const CONFIG_BOOL_TABLE bool_defaults[] = {
 	VAR_DISABLE_DNS, DEF_DISABLE_DNS, &var_disable_dns,
 	VAR_SOFT_BOUNCE, DEF_SOFT_BOUNCE, &var_soft_bounce,
 	VAR_OWNREQ_SPECIAL, DEF_OWNREQ_SPECIAL, &var_ownreq_special,
@@ -592,6 +621,7 @@ void    mail_params_init()
 	VAR_OLDLOG_COMPAT, DEF_OLDLOG_COMPAT, &var_oldlog_compat,
 	VAR_HELPFUL_WARNINGS, DEF_HELPFUL_WARNINGS, &var_helpful_warnings,
 	VAR_CYRUS_SASL_AUTHZID, DEF_CYRUS_SASL_AUTHZID, &var_cyrus_sasl_authzid,
+	VAR_MULTI_ENABLE, DEF_MULTI_ENABLE, &var_multi_enable,
 	0,
     };
     const char *cp;
@@ -658,6 +688,7 @@ void    mail_params_init()
 #ifdef HAS_DB
     dict_db_cache_size = var_db_read_buf;
 #endif
+    inet_windowsize = var_inet_windowsize;
 
     /*
      * Variables whose defaults are determined at runtime, after other
