@@ -146,24 +146,24 @@ int     smtpd_sasl_auth_cmd(SMTPD_STATE *state, int argc, SMTPD_TOKEN *argv)
 	smtpd_chat_reply(state, "503 5.5.1 Error: send HELO/EHLO first");
 	return (-1);
     }
-    if (SMTPD_STAND_ALONE(state) || !var_smtpd_sasl_enable
+    if (SMTPD_STAND_ALONE(state) || !smtpd_sasl_is_active(state)
 	|| (state->ehlo_discard_mask & EHLO_MASK_AUTH)) {
 	state->error_mask |= MAIL_ERROR_PROTOCOL;
 	smtpd_chat_reply(state, "503 5.5.1 Error: authentication not enabled");
 	return (-1);
     }
     if (smtpd_milters != 0 && (err = milter_other_event(smtpd_milters)) != 0) {
-        if (err[0] == '5') {
-            state->error_mask |= MAIL_ERROR_POLICY;
-            smtpd_chat_reply(state, "%s", err);
-            return (-1);
-        }
-        /* Sendmail compatibility: map 4xx into 454. */
-        else if (err[0] == '4') {
-            state->error_mask |= MAIL_ERROR_POLICY;
-            smtpd_chat_reply(state, "454 4.3.0 Try again later");
-            return (-1);
-        }
+	if (err[0] == '5') {
+	    state->error_mask |= MAIL_ERROR_POLICY;
+	    smtpd_chat_reply(state, "%s", err);
+	    return (-1);
+	}
+	/* Sendmail compatibility: map 4xx into 454. */
+	else if (err[0] == '4') {
+	    state->error_mask |= MAIL_ERROR_POLICY;
+	    smtpd_chat_reply(state, "454 4.3.0 Try again later");
+	    return (-1);
+	}
     }
 #ifdef USE_TLS
     if (state->tls_auth_only && !state->tls_context) {
@@ -184,27 +184,6 @@ int     smtpd_sasl_auth_cmd(SMTPD_STATE *state, int argc, SMTPD_TOKEN *argv)
 	return (-1);
     }
 
-    /* Don't reuse the SASL handle after authentication failure. */
-#ifndef SMTPD_FLAG_AUTH_USED
-#define SMTPD_FLAG_AUTH_USED	(1<<15)
-#endif
-#ifndef XSASL_TYPE_CYRUS
-#define XSASL_TYPE_CYRUS	"cyrus"
-#endif
-    if (state->flags & SMTPD_FLAG_AUTH_USED) {
-	smtpd_sasl_disconnect(state);
-#ifdef USE_TLS
-	if (state->tls_context != 0)
-	    smtpd_sasl_connect(state, VAR_SMTPD_SASL_TLS_OPTS,
-			       var_smtpd_sasl_tls_opts);
-	else
-#endif
-	    smtpd_sasl_connect(state, VAR_SMTPD_SASL_OPTS,
-			       var_smtpd_sasl_opts);
-    } else if (strcmp(var_smtpd_sasl_type, XSASL_TYPE_CYRUS) == 0) {
-	state->flags |= SMTPD_FLAG_AUTH_USED;
-    }
-
     /*
      * All authentication failures shall be logged. The 5xx reply code from
      * the SASL authentication routine triggers tar-pit delays, which help to
@@ -230,7 +209,7 @@ char   *smtpd_sasl_mail_opt(SMTPD_STATE *state, const char *addr)
     /*
      * Do not store raw RFC2554 protocol data.
      */
-    if (!var_smtpd_sasl_enable) {
+    if (!smtpd_sasl_is_active(state)) {
 	state->error_mask |= MAIL_ERROR_PROTOCOL;
 	return ("503 5.5.4 Error: authentication disabled");
     }
